library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity FMTransmitter is
	port (
		Clk         : in std_logic;
		Reset       : in std_logic;
		Button440   : in std_logic;
		Button880   : in std_logic;
		AntOut      : out std_logic
	);
end entity;

architecture Architecture1 of FMTransmitter is
	component Oscillator is
		generic (
			NBits : integer := 32
		);
		port (
			IncrementIn : in std_logic_vector(31 downto 0);
			Clk         : in std_logic;
			Reset       : in std_logic;
			DataOut     : out std_logic
		);
	end component;
	component Mux2Inputs is
		generic (
			NBits : integer := 32
		);
		port (
			Sel     : in std_logic;
			DataIn0 : in std_logic_vector((NBits - 1) downto 0);
			DataIn1 : in std_logic_vector((NBits - 1) downto 0);
			DataOut : out std_logic_vector((NBits - 1) downto 0)
		);
	end component;
	signal Mux1Out : std_logic_vector(31 downto 0);
	signal Mux1Sel : std_logic;
	signal ConvOut : std_logic_vector(31 downto 0);
	signal ConvIn  : std_logic;
	signal Mux2Out : std_logic_vector(31 downto 0);
begin
	RadioOsc : Oscillator generic map (
		NBits => 32
	)
	port map (
		Clk => Clk,
		Reset => Reset,
		IncrementIn => Mux1Out,
		DataOut => AntOut
	);
	Mux1Sel <= Button440 or Button880;
	Mux1 : Mux2Inputs generic map (
		NBits => 32
	)
	port map (
		Sel => Mux1Sel,
		DataIn0 => std_logic_vector(to_unsigned(1342177280, 32)),   -- center freq = 90.0 MHz
		DataIn1 => ConvOut,
		DataOut => Mux1Out
	);
	-- center freq - 75 KHz when 0
	-- center freq + 75 KHz when 1
	ConvOut <= std_logic_vector(to_unsigned(1341058799, 32)) when (ConvIn = '0') else
	           std_logic_vector(to_unsigned(1343295761, 32));
	AudioOsc : Oscillator generic map (
		NBits => 32
	)
	port map (
		Clk => Clk,
		Reset => Reset,
		IncrementIn => Mux2Out,
		DataOut => ConvIn
	);
	Mux2 : Mux2Inputs generic map (
		NBits => 32
	)
	port map (
		Sel => Button880,
		DataIn0 => std_logic_vector(to_unsigned(6562, 32)),   -- 440 Hz
		DataIn1 => std_logic_vector(to_unsigned(13124, 32)),  -- 880 Hz
		DataOut => Mux2Out
	);
end architecture;
