library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_signed.all;

-- IncrementIn = (desired_output_frequency / clk_frequency) * 2^32
--
-- clk_frequency = 320 MHz
-- desired_output_frequency = 100 MHz
-- FreqIn = 1342177280


entity Oscillator is
	generic (
		NBits : integer := 32
	);
	port (
		IncrementIn : in std_logic_vector(31 downto 0);
		Clk         : in std_logic;
		Reset       : in std_logic;
		DataOut     : out std_logic
	);
end entity;

architecture Architecture1 of Oscillator is
	component Reg is
		generic (
			NBits : integer := 32
		);
		port (
			Enable  : in std_logic;
			Clk     : in std_logic;
			DataIn  : in std_logic_vector((NBits - 1) downto 0);
			DataOut : out std_logic_vector((NBits - 1) downto 0)
		);
	end component;
	signal MuxOut : std_logic_vector((NBits - 1) downto 0);
	signal RegOut : std_logic_vector((NBits - 1) downto 0);
begin
	PhaseAcc : Reg generic map (
		NBits => NBits
	)
	port map (
		Enable => '1',
		Clk => Clk,
		DataIn => MuxOut,
		DataOut => RegOut
	);
	MuxOut <= (others => '0') when (Reset = '1') else
	          (RegOut + IncrementIn);
	DataOut <= RegOut(NBits - 1);
end architecture;
